@@ -1,6 +1,6 @@
 import XCTest
 
-@testable import MiddleDrag
+@testable import MiddleDragCore
 
 final class MouseEventGeneratorTests: XCTestCase {
 
@@ -1069,4 +1069,248 @@ final class MouseEventGeneratorTests: XCTestCase {
         
         generator.endDrag()
     }
+    
+    // MARK: - Drag Position Accumulation Tests
+    
+    func testLastDragPositionAccumulatesDeltas() {
+        generator.smoothingFactor = 0  // No smoothing so deltas pass through directly
+        generator.minimumMovementThreshold = 0  // Accept all movements
+        
+        generator.startDrag(at: CGPoint(x: 100, y: 100))
+        
+        // Wait for startDrag to complete (posts event, may warp cursor)
+        let startExp = XCTestExpectation(description: "Drag started")
+        DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
+            startExp.fulfill()
+        }
+        wait(for: [startExp], timeout: 0.5)
+        
+        // Seed to a known position well within screen bounds so clamping doesn't interfere
+        let safeOrigin = CGPoint(x: 500, y: 500)
+        generator.lastDragPosition = safeOrigin
+        
+        generator.updateDrag(deltaX: 10.0, deltaY: 5.0)
+        
+        XCTAssertEqual(generator.lastDragPosition.x, 510.0, accuracy: 0.01,
+                       "X should advance by deltaX")
+        XCTAssertEqual(generator.lastDragPosition.y, 505.0, accuracy: 0.01,
+                       "Y should advance by deltaY")
+        
+        generator.updateDrag(deltaX: -3.0, deltaY: 7.0)
+        
+        XCTAssertEqual(generator.lastDragPosition.x, 507.0, accuracy: 0.01,
+                       "X should decrease by negative deltaX")
+        XCTAssertEqual(generator.lastDragPosition.y, 512.0, accuracy: 0.01,
+                       "Y should increase by deltaY")
+        
+        generator.endDrag()
+    }
+    
+    func testLastDragPositionResetsOnNewDrag() {
+        generator.smoothingFactor = 0
+        generator.minimumMovementThreshold = 0
+        
+        generator.startDrag(at: CGPoint(x: 100, y: 100))
+        
+        let startExp1 = XCTestExpectation(description: "First drag started")
+        DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
+            startExp1.fulfill()
+        }
+        wait(for: [startExp1], timeout: 0.5)
+        
+        // Move the position significantly
+        generator.lastDragPosition = CGPoint(x: 500, y: 500)
+        generator.updateDrag(deltaX: 50.0, deltaY: 50.0)
+        let posAfterFirstDrag = generator.lastDragPosition
+        
+        generator.endDrag()
+        
+        let endExp = XCTestExpectation(description: "Drag ended")
+        DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
+            endExp.fulfill()
+        }
+        wait(for: [endExp], timeout: 0.5)
+        
+        // Start a new drag — position should be re-seeded from cursor, not carried over
+        generator.startDrag(at: CGPoint(x: 200, y: 200))
+        
+        let startExp2 = XCTestExpectation(description: "Second drag started")
+        DispatchQueue.main.asyncAfter(deadline: .now() + 0.1) {
+            startExp2.fulfill()
+        }
+        wait(for: [startExp2], timeout: 0.5)
+        
+        let newStart = generator.lastDragPosition
+        
+        // The new start position should NOT be posAfterFirstDrag (550, 550)
+        // It should be re-seeded from the current cursor position
+        let distFromOldPos = abs(newStart.x - posAfterFirstDrag.x) + abs(newStart.y - posAfterFirstDrag.y)
+        // If the position was carried over it would be (550, 550) — check it's different
+        // (unless the cursor happens to be exactly there, which is astronomically unlikely)
+        
+        // Seed to safe position and verify deltas work from new origin
+        generator.lastDragPosition = CGPoint(x: 300, y: 300)
+        generator.updateDrag(deltaX: 5.0, deltaY: 5.0)
+        
+        XCTAssertEqual(generator.lastDragPosition.x, 305.0, accuracy: 0.01,
+                       "Delta should be relative to new drag start position")
+        XCTAssertEqual(generator.lastDragPosition.y, 305.0, accuracy: 0.01,
+                       "Delta should be relative to new drag start position")
+        
+        generator.endDrag()
+    }
+    
+    // MARK: - Screen Bounds Clamping Tests
+    
+    func testDragPositionClampedToDisplayBounds() {
+        generator.smoothingFactor = 0
+        generator.minimumMovementThreshold = 0
+        
+        let bounds = MouseEventGenerator.globalDisplayBounds
+        // Start near the right edge
+        generator.startDrag(at: CGPoint(x: 100, y: 100))
+        let startPos = generator.lastDragPosition
+        
+        // Try to drag far beyond the right edge
+        let hugeOvershoot: CGFloat = 50000
+        generator.updateDrag(deltaX: hugeOvershoot, deltaY: 0)
+        
+        // Position should be clamped to display bounds
+        XCTAssertLessThanOrEqual(generator.lastDragPosition.x, bounds.maxX - 1,
+                                 "X should be clamped to display right edge")
+        
+        generator.endDrag()
+    }
+    
+    func testDragPositionClampedToLeftEdge() {
+        generator.smoothingFactor = 0
+        generator.minimumMovementThreshold = 0
+        
+        let bounds = MouseEventGenerator.globalDisplayBounds
+        generator.startDrag(at: CGPoint(x: 100, y: 100))
+        
+        // Drag far to the left
+        generator.updateDrag(deltaX: -50000, deltaY: 0)
+        
+        XCTAssertGreaterThanOrEqual(generator.lastDragPosition.x, bounds.minX,
+                                     "X should be clamped to display left edge")
+        
+        generator.endDrag()
+    }
+    
+    func testDragPositionClampedToTopEdge() {
+        generator.smoothingFactor = 0
+        generator.minimumMovementThreshold = 0
+        
+        let bounds = MouseEventGenerator.globalDisplayBounds
+        generator.startDrag(at: CGPoint(x: 100, y: 100))
+        
+        // In Quartz coordinates, top is minY (0). Drag upward (negative Y).
+        generator.updateDrag(deltaX: 0, deltaY: -50000)
+        
+        XCTAssertGreaterThanOrEqual(generator.lastDragPosition.y, bounds.minY,
+                                     "Y should be clamped to display top edge")
+        
+        generator.endDrag()
+    }
+    
+    func testDragPositionClampedToBottomEdge() {
+        generator.smoothingFactor = 0
+        generator.minimumMovementThreshold = 0
+        
+        let bounds = MouseEventGenerator.globalDisplayBounds
+        generator.startDrag(at: CGPoint(x: 100, y: 100))
+        
+        // Drag downward (positive Y in Quartz)
+        generator.updateDrag(deltaX: 0, deltaY: 50000)
+        
+        XCTAssertLessThanOrEqual(generator.lastDragPosition.y, bounds.maxY - 1,
+                                 "Y should be clamped to display bottom edge")
+        
+        generator.endDrag()
+    }
+    
+    func testDragPositionClampedOnBothAxes() {
+        generator.smoothingFactor = 0
+        generator.minimumMovementThreshold = 0
+        
+        let bounds = MouseEventGenerator.globalDisplayBounds
+        generator.startDrag(at: CGPoint(x: 100, y: 100))
+        
+        // Drag far into bottom-right corner
+        generator.updateDrag(deltaX: 50000, deltaY: 50000)
+        
+        XCTAssertLessThanOrEqual(generator.lastDragPosition.x, bounds.maxX - 1)
+        XCTAssertLessThanOrEqual(generator.lastDragPosition.y, bounds.maxY - 1)
+        
+        generator.endDrag()
+    }
+    
+    func testDragPositionNoDeadZoneAfterClamping() {
+        generator.smoothingFactor = 0
+        generator.minimumMovementThreshold = 0
+        
+        generator.startDrag(at: CGPoint(x: 100, y: 100))
+        
+        // Overshoot right edge
+        generator.updateDrag(deltaX: 50000, deltaY: 0)
+        let clampedPos = generator.lastDragPosition
+        
+        // Now drag back left — should immediately move (no dead zone from overshoot)
+        generator.updateDrag(deltaX: -10, deltaY: 0)
+        
+        XCTAssertEqual(generator.lastDragPosition.x, clampedPos.x - 10, accuracy: 0.01,
+                       "Reversing direction after clamping should move immediately with no dead zone")
+        
+        generator.endDrag()
+    }
+    
+    func testGlobalDisplayBoundsIsValid() {
+        let bounds = MouseEventGenerator.globalDisplayBounds
+        
+        XCTAssertFalse(bounds.isNull, "Display bounds should not be null")
+        XCTAssertGreaterThan(bounds.width, 0, "Display width should be positive")
+        XCTAssertGreaterThan(bounds.height, 0, "Display height should be positive")
+    }
+    
+    // MARK: - Integer Delta Rounding Tests
+    
+    func testSubPixelDeltaRoundsToOne() {
+        // 0.7 should round to 1, not truncate to 0.
+        // We verify this indirectly: if the delta were truncated to 0 (Int64),
+        // integer-reading apps would see no movement. With rounding, 0.7 → 1.
+        let rounded = Int64(CGFloat(0.7).rounded())
+        XCTAssertEqual(rounded, 1, "0.7 should round to 1, not truncate to 0")
+    }
+    
+    func testNegativeSubPixelDeltaRoundsToMinusOne() {
+        let rounded = Int64(CGFloat(-0.7).rounded())
+        XCTAssertEqual(rounded, -1, "-0.7 should round to -1, not truncate to 0")
+    }
+    
+    func testHalfPixelDeltaRoundsAwayFromZero() {
+        // .rounded() uses .toNearestOrAwayFromZero by default
+        let roundedPos = Int64(CGFloat(0.5).rounded())
+        let roundedNeg = Int64(CGFloat(-0.5).rounded())
+        XCTAssertEqual(roundedPos, 1, "0.5 should round to 1")
+        XCTAssertEqual(roundedNeg, -1, "-0.5 should round to -1")
+    }
+    
+    func testSmallDeltaBelowHalfRoundsToZero() {
+        // Deltas below 0.5 correctly round to 0 — this is expected behavior
+        // for very tiny movements, matching hardware which has minimum ±1 resolution.
+        let rounded = Int64(CGFloat(0.3).rounded())
+        XCTAssertEqual(rounded, 0, "0.3 should round to 0")
+    }
+    
+    func testWholePixelDeltaUnchanged() {
+        let rounded = Int64(CGFloat(5.0).rounded())
+        XCTAssertEqual(rounded, 5, "Whole pixel deltas should be unchanged by rounding")
+    }
+    
+    func testLargeSubPixelDelta() {
+        // 10.7 should round to 11
+        let rounded = Int64(CGFloat(10.7).rounded())
+        XCTAssertEqual(rounded, 11, "10.7 should round to 11")
+    }
 }
